export declare abstract class AddressBase {
    protected privateData: string;
    constructor(data: string);
    /**
     * Exposing the private data for the derived classes (dev only/python like privatization)
     */
    get _data(): string;
    /**
     * Return the well defined standardized address hash for address created by the string representation used at the constructor
     * @returns {string} The standardized address hash (32 bytes hex string without prefix 0x)
     */
    abstract get stdHash(): string;
    /**
     * Returns the type of address
     */
    abstract get type(): string;
    /**
     *Returns true if the address is valid on main
     */
    abstract get isMainnet(): boolean;
    /**
     * Check if address has the correct format and satisfies checksum
     * Some chains support multiple types of addresses that have different address format definitions (ie X addresses and classic addresses)
     */
    abstract isValid(): boolean;
    /**
     * Static method to convert a string to a standard hash
     * @param a 0x prefixed string to be hashed
     * @returns standardized un-prefixed keccak256 hash
     */
    protected static toStandardHash(a: string): string;
}
export { XrpAddress } from "./addressObjects/XrpAddress";
//# sourceMappingURL=AddressBase.d.ts.map